local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local LSM = LibStub("LibSharedMedia-3.0")

local Boss = RW.bossPrototype
local messageProt = {}
local mt = {__index = messageProt}

local Party = RW:GetPlugin("Raid")
local BITS = RW.BITS

Boss.BITS = Boss.BITS or {}
Boss.BITS.MESSAGE = {
	NORMAL	= 0x00000001,
	CAST 	= 0x00000002,
	TARGET 	= 0x00000004,
	TARGET_STACK	= 0x00000008,
	PHASE	= 0x00000010,
}

-- If text is a number this warning is considered not a custom one
-- If text is not a number then it is custom and it can be possibly sent as a table,
-- then the first value in the table will be the actual text, and the second oen will be the option text as its show in the GUI
-- 3rd value in the table will be the tooltip text
local custText = true
function Boss:_Message(mType, text, icon, optionName, default, color)
	if type(text) == "number" then 
		custText = false
		return self:_Message(mType, GetSpellInfo(text), icon or text, optionName or text, default, color) 
	end
	if type(icon) == "number" then
		return self:_Message(mType, text, select(3, GetSpellInfo(icon)), optionName or icon, default, color)
	end
	-- If option name is a number, then add it as a spell option and update it to the actual name
	-- Else add a gereral option
	if type(optionName) == "number" then
		self:AddSpellOption("General", optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName or text)
	end
	-- If no option name is found by now we use the text as default
	if not optionName then optionName = type(text) == "table" and text[1] or text end
	
	local customOptName
	if type(text) == "table" then
		customOptName = text[2]
		text = text[1]
	elseif custText then
		customOptName = text
	end
	
	
	self:AddDefaults(optionName, "MESSAGE", mType, default, customOptName)
	
	local defaultOption = self.defaultOptions[optionName]
	defaultOption.Colors = defaultOption.Colors or {}
	defaultOption.Sounds = defaultOption.Sounds or {}
	defaultOption.Colors[customOptName or "MESSAGE"..optionName] = color
	defaultOption.Sounds[customOptName or "MESSAGE"..optionName] = "RW: Click"
	
	obj = setmetatable({
		type = mType,
		text = text,
		customText = custText,
		customOptName = customOptName,
		icon = icon,
		id = optionName,
		scheduled = {},
		mod = self,
	}, mt)
	
	self.messages = self.messages or {}
	tinsert(self.messages, obj)
	
	custText = true
	
	return obj
end

function Boss:Message(...)
	return self:_Message("NORMAL", ...)
end

function Boss:CastMessage(time, ...)
	return self:_Message("CAST", time, ...)
end

function Boss:TargetMessage(...)
	return self:_Message("TARGET", ...)
end

function Boss:TargetStackMessage(...)
	return self:_Message("TARGET_STACK", ...)
end

function Boss:PhaseMessage(...)
	return self:_Message("PHASE", ...)
end

function Boss:CustomMessage(...)
	return self:_Message("CUSTOM", ...)
end

function messageProt:Show(...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.MESSAGE) ~= RW.BITS.MESSAGE then return end
	if self.customOptName then
		if not self.mod.db.profile[self.id]["MESSAGE"..self.customOptName] then return end
	elseif bit.band(self.mod.db.profile[self.id].MESSAGE, self.mod.BITS.MESSAGE[self.type]) ~= self.mod.BITS.MESSAGE[self.type] then return end
	
	local text
	if self.customText then		
		text = RW:Format(self.text, ...)
	else
		text = RW:Format(L["FORMAT_MESSAGE_"..self.type], self.text, ...)
	end
	text = text:gsub(">([%S\s]+)<", function(name)
		if Party.group[name] then
			local color = RAID_CLASS_COLORS[Party.group[name].class] or self.color
			name = ("|cff%.2x%.2x%.2x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
		end
		return name
	end)
	local tc = self.mod.db.profile[self.id].Colors[self.customOptName or "MESSAGE"..self.id]
	local c 
	if tc then c = tc else c = {1, 1, 1, 1} end
	self.mod:SendMessage("RW_Message", text, c[1], c[2], c[3], c[4], self.icon)
	self.mod:Msg((self.icon and "|T"..self.icon..":20|t " or "")..text)
	PlaySoundFile(LSM:Fetch("sound", self.mod.db.profile[self.id].Sounds[self.customOptName or "MESSAGE"..self.id]))
end
messageProt.Start = messageProt.Show

local function showMessage(message, ...)
	message:Show(...)
end

function messageProt:Schedule(stime, info, ...)
	self.mod:Schedule("MESSAGE"..self.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.scheduled, "MESSAGE"..self.id..(info or ""))
end

function messageProt:ScheduleRepeating(stime, now, info, ...)
	self.mod:ScheduleRepeating("MESSAGE"..self.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.scheduled, "MESSAGE"..self.id..(info or ""))
end

function messageProt:Unschedule(extraID)
	if not extraID then
		for i, messageID in pairs(self.scheduled) do
			self.mod:Unschedule(messageID)
		end
		wipe(self.scheduled)
	else
		self.mod:Unschedule("MESSAGE"..self.id..(extraID or ""))
		RW:TRemove(self.scheduled, "MESSAGE"..self.id..(extraID or ""))
	end
end
messageProt.Cancel = messageProt.Unschedule

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.MESSAGE(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")
	
	for i, message in pairs(self.messages) do
		if message.id == option.text then
			local subGroup = AceGUI:Create("SimpleGroup")
			subGroup:SetFullWidth(true)
			subGroup:SetLayout("flow")
			
			local check = AceGUI:Create("CheckBox")
			if message.customOptName then
				check:SetLabel(message.customOptName)
				check:SetValue(self.db.profile[option.text]["MESSAGE"..message.customOptName])
				check:SetUserData("optName", "MESSAGE"..message.customOptName)
				check:SetCallback("OnValueChanged", self.optionHelpers.onCustomValueChanged)
			else
				check:SetLabel(L["MESSAGE_"..message.type]) -- TODO: Set more info in the text, (the actual time for the MESSAGE)
				check:SetValue(bit.band(self.db.profile[option.text].MESSAGE, Boss.BITS.MESSAGE[message.type]) == Boss.BITS.MESSAGE[message.type])
				check:SetUserData("type", "MESSAGE")
				check:SetUserData("subType", message.type)
				check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)
				check:SetCallback("OnEnter", self.optionHelpers.onOptionEnter)
			end			
			check:SetUserData("option", option)			
			check:SetUserData("boss", self)			
			check:SetCallback("OnLeave", self.optionHelpers.onOptionLeave)
			
			local color = AceGUI:Create("ColorPicker")
			local colorvalue = self.db.profile[option.text].Colors[message.customOptName or "MESSAGE"..message.id]
			if colorvalue then 
				color:SetColor(unpack(colorvalue))
			end
			color:SetLabel("Color")
			color:SetUserData("option", option)
			color:SetUserData("boss", self)
			color:SetUserData("key", "Colors")
			color:SetUserData("subKey", message.customOptName or "MESSAGE"..message.id)
			color:SetCallback("OnValueConfirmed", self.optionHelpers.onValueChanged2)
			
			local drop = AceGUI:Create("LSM30_Sound")
			drop:SetList()
			--drop:SetList(AceGUIWidgetLSMlists.sound)
			drop:SetLabel("Sound")
			drop:SetUserData("option", option)
			drop:SetUserData("boss", self)
			drop:SetUserData("key", "Sounds")
			drop:SetUserData("subKey", message.customOptName or "MESSAGE"..message.id)
			drop:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged2)
			drop:SetValue(self.db.profile[option.text].Sounds[message.customOptName or "MESSAGE"..message.id])	
			
			subGroup:AddChild(check)
			subGroup:AddChild(color)
			subGroup:AddChild(drop)
			
			group:AddChild(subGroup)
		end
	end
	return group
end 